/* Utility to test the erandom sysctl interface */

#include <stdio.h>
#include <stdlib.h>
#include <sys/sysctl.h>
#include <linux/sysctl.h>
#include <linux/version.h>

#if LINUX_VERSION_CODE < 0x020120
#error "This program needs linux-2.1.32 or newer to compile"
#endif

// Uncomment thw following line, if RANDOM_ERANDOM isn't defined, and you
// don't feel like fixing /usr/include/linux/sysctl.h (see INSTALL file)
// #define RANDOM_ERANDOM 7

int main(int argc, char **argv)
{
	int name[] = {CTL_KERN, KERN_RANDOM, RANDOM_ERANDOM};
        int namelen = 3;
	unsigned char buf[16];
	size_t len = sizeof(buf);
	int i, error;
	FILE *file;

/* We try to open /proc/sys/kernel/random/erandom. If we fail, we can forget
   about even trying sysctl.

   Note that we don't use the file, but we're only interested if it exists.
 */

	if ( (file = fopen("/proc/sys/kernel/random/erandom", "r")) == NULL) {
		fprintf(stderr, "Failed to open /proc/sys/kernel/random/erandom. It seems\n"
			"like you don't have frandom compiled into the kernel. (module\n"
			"inclusion isn't enough)\n");
		exit(1);
	}

	fclose(file);

	error = sysctl (name, namelen, (void *)buf, &len,
			NULL /* newval */, 0 /* newlen */);
	if (error) {
		fprintf(stderr,"%s: sysctl(): %s\n",
			argv[0],strerror(error));
		exit(1);
	}
	
	if (len != 16) {
		fprintf(stderr,"len is %i bytes and not 16!\n", len);
		exit(1);
	}
	
	for (i=0; i<16; i++)
		printf("%02x ", buf[i]);	
	printf("\n");
	
	return 0;
}

